#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import uuid
import shutil
import zipfile
import logging
import subprocess
from hashlib import md5
from pathlib import Path

import rarfile

from ezsub import const
from ezsub.utils import to_screen
from ezsub.errors import (
    EmptyArchiveError,
    BadCompressedFileError,
    NothingToExtractError
)

logger = logging.getLogger(__name__)
cur = const.Curser

def get_extracor(file):
    if zipfile.is_zipfile(file):
        return zipfile.ZipFile
    elif rarfile.is_rarfile(file):
        return rarfile.RarFile
    else:
        raise BadCompressedFileError


class Destination(object):
    def __init__(self, root=const.DESTINATION, group=const.GROUP, open_after=const.OPEN_AFTER):
        self.root = Path(root)
        self.group = group
        self.open_after = open_after

    def extract(self, to_extract):
        if not to_extract:
            logger.warn("nothing to extract")
            raise NothingToExtractError
        empty_files = 0
        unrar_missing = False
        bad_files = list()
        destinations = set()
        n = len(to_extract)
        for i, item in enumerate(to_extract):
            to_screen(f"\rextracting... {i+1}/{n}", end='')  # progress
            file = item['path']
            dest = self.make_destination(file)
            destinations.add(dest)
            try:
                extractor = get_extracor(file)
                _extract(file, extractor, dest)
            except EmptyArchiveError:
                empty_files += 1
            except BadCompressedFileError:
                name = '-'.join(item['path'].parts[-3:])
                item['dest'] = dest.joinpath(name).with_suffix('.txt')
                bad_files.append(item)
            except rarfile.RarCannotExec:
                unrar_missing = True
        else:
            to_screen("\rextracting... ", end='')
            to_screen(f"{cur.CFH}done!", style="ok")

        complain(empty_files, destinations, unrar_missing, bad_files)
        self.open()
        return None

    def open(self):
        if not self.root.exists():
            logger.warn("destination path '%s' not exists to open. Ignored.", self.root.resolve())
            return None

        if not self.open_after:
            return None

        path = str(self.root.resolve())
        if const.OS == 'Windows':
            app = 'explorer',
        elif const.OS == 'Darwin':
            app = 'open',
        else:
            app = 'xdg-open'
        to_screen("opening destination... ", end='')
        subprocess.Popen([app, path])
        to_screen(f"done!", style="ok")
        return None

    def make_destination(self, filepath):
        dest = self.root
        if self.group:
            dest = self.root.joinpath(*filepath.parts[-3:-1])
        dest.mkdir(parents=True, exist_ok=True)
        return dest

    def get_child(self, child, mk_parents=False):
        if str(child).startswith('/'):
            child = f".{child}"  # make it relative
        path = self.root.joinpath(child)
        if mk_parents:
            path.parent.mkdir(parents=True, exist_ok=True)
        return path.resolve()


def _extract(file, extractor, dest):
    with extractor(file) as archive:
        if archive.namelist():
            extract_members(dest, file, archive)
        else:
            raise EmptyArchiveError
    return None


def extract_members(dest, file, archive):
    for member in archive.infolist():
        if hasattr(member, 'is_dir'):  # zipfile
            if member.is_dir():
                continue
        elif hasattr(member, 'isdir'):  # rarfile
            if member.isdir():
                continue
        extracted_file = dest.joinpath(member.filename)
        try:
            extracted_file.exists()
        except OSError:
            # windows illegal characters in file name raise this error. skip this file.
            continue
        preferred_name = get_name(file, dest, member.filename)
        archive.extract(member.filename, str(dest))
        extracted_file.rename(preferred_name)
    return None


def get_language(file_path):
    language = file_path.parent.name
    if language == 'farsi_persian':
        language = 'farsi'
    return language


def get_name(file, dest, item):
    filename = dest.joinpath(item)
    language = get_language(file)
    name_options = {
        "1": filename,
        "2": filename.with_suffix(f'.{language}{filename.suffix}'),
        "3": filename.parent.joinpath(f"{file.stem}.{language}.{filename.name}")
    }
    preferred = "2"  # TODO: add argument to get name pattern from user
    filename = name_options[preferred]
    if filename.exists():
        _rand = str(uuid.uuid4())[0:3]
        new_name = filename.with_suffix(f'.{_rand}{filename.suffix}')
        filename.rename(new_name)

    return filename


def complain(empty=False, dest_list=None, no_unrar=None, bad_files=None):
    if empty:
        to_screen("found empty archives: ", end='')
        to_screen(empty, style="warn")

    files_removed = remove_same(dest_list)
    if files_removed:
        to_screen("removed duplicate subtitles: ", end='')
        to_screen(files_removed, style="warn")

    if no_unrar:
        to_screen("Some rar archives are found but unrar executable is missing.", style="warn")

    if bad_files:
        to_screen("Some files are not zip nor rar.So these are considered as text files:", style="warn")
        for item in bad_files:
            to_screen(f"       {item['path']}", style="info")
            if item['url']:
                to_screen(f"           downloaded from: {item['url']}", style="info")
            shutil.copy2(str(item['path']), str(item['dest']))


def remove_same(paths):
    if not paths:
        return 0
    to_remove = []
    for path in paths:
        # TODO: skip for known media types
        files = [
            {'file': item, 'md5': md5(item.read_bytes()).digest()}
            for item in path.iterdir() if item.is_file()
        ]
        i = 0
        for file1 in files:
            for file2 in files[i+1:]:
                if file1['md5'] == file2['md5']:
                    to_remove.append(file1['file'])
                    break
            i += 1

    for file in to_remove:
        file.unlink()

    return len(to_remove)


# TODO: use this function in case of illegal characters
def test_path(folder, name):
    try:
        filepath = folder.joinpath(name)
        filepath.exists()
    except OSError:
        # windows has some limitations on file names.
        # '/' might be in namelist because some compressed file have folders.
        illegals = r'<>:"\|?*'
        name = "".join([c for c in name if c not in illegals]).rstrip()
    return name



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ezsub.cache import Cache, prune
from ezsub.errors import NoResultError, NothingToCleanError
from ezsub.utils import to_screen, select, parse_lngs, get_size, human_readable


def clean(req):
    cache = Cache()
    results = cache.search(req.title)
    if not results:
        raise NoResultError

    selected = select(results, req.auto_select)
    paths = [results[s-1]['path'] for s in selected]
    lngs = parse_lngs(req.lngs)
    to_clean = prune(paths, lngs)
    if not to_clean:
        raise NothingToCleanError

    files = [item['path'] for item in to_clean]

    size_before = get_size(cache.subtitles)
    if req.zero:
        action = cache.zero(files)
    else:
        action = cache.delete(files)

    cache.delete_empty_folders()
    size_after = get_size(cache.subtitles)
    to_screen(f"\n{human_readable(size_before-size_after)} freed by {action} files.\n", style='warn')

    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import logging
import argparse
from pathlib import Path
from ezsub import const
from ezsub.conf import UserConf


logger = logging.getLogger(__name__)

SORT_OPTIONS = ['t', 's', 'n']
# common arguments as a parser
destination = argparse.ArgumentParser(add_help=False)
destination.add_argument(
    '-d',
    '--destination',
    dest='destination',
    metavar="DESTINATION",
    help='Destination folder.')

title = argparse.ArgumentParser(add_help=False)
title.add_argument(
    '-t',
    '--title',
    dest='title',
    metavar="TITLE",
    nargs='+',
    help='Title to search')

lngs = argparse.ArgumentParser(add_help=False)
lngs.add_argument(
    '-l',
    '--lngs',
    dest='lngs',
    nargs='+',
    choices=const.SUPPORTED_LNGS,
    metavar=('LNG1', 'LNG2',),
    help='Language(s) which you are interested in.')

site = argparse.ArgumentParser(add_help=False)
site.add_argument(
    '-s',
    '--site',
    dest='site',
    nargs='+',
    choices=const.MIRRORS,
    metavar=('Site1', 'Site2'),
    help='Site to download from.')


verbose = argparse.ArgumentParser(add_help=False)
verbose.add_argument(
    '-v',
    dest='verbosity',
    action='count',
    help='increase verbosity. use -v, -vv, -vvv ...')

group = argparse.ArgumentParser(add_help=False)
_group = group.add_mutually_exclusive_group()
_group.add_argument(
    '-g',
    dest='group',
    action='store_true',
    help="Extract subtitles to folders like './destination/title/language'")
_group.add_argument(
    '-G',
    dest='group',
    action='store_false',
    help='Extract all subtitles to the root of the destination')

ask = argparse.ArgumentParser(add_help=False)
_ask = ask.add_mutually_exclusive_group()
_ask.add_argument(
    '-a',
    '--auto-select',
    dest='auto_select',
    action='store_true',
    help='Select first title in search results automatically.')
_ask.add_argument(
    '-A',
    '--ask',
    dest='auto_select',
    action='store_false',
    help='Ask user to select best match in search results.')

open_after = argparse.ArgumentParser(add_help=False)
_open = open_after.add_mutually_exclusive_group()
_open.add_argument(
    '-o',
    '--open-after',
    dest='open_after',
    action='store_true',
    help='Open destination folder after.')
_open.add_argument(
    '-O',
    '--no-open-after',
    dest='open_after',
    action='store_false',
    help='Do not open destination folder after.')

examples = f'''
Examples:
{const.PROGRAMNAME} dl -t riverdale third season -l fa en -a
'''


def get_parser():
    parser = argparse.ArgumentParser(
        prog=const.PROGRAMNAME,
        description='Downloads subtitles from subscene and its mirrors',
        formatter_class=argparse.RawTextHelpFormatter,
        add_help=True,
        epilog=examples)

    parser.add_argument(
        "--version",
        action="version",
        version=f'%(prog)s version {const.__version__}',
        help="show version and exit")

    commands = parser.add_subparsers(
        title='commands',
        dest="command",
        required=True,
        metavar='COMMAND')

    download = commands.add_parser(
        'download',
        parents=[title, destination, _open, _ask, _group, site, lngs],
        aliases=['d', 'dl'],
        help='Download subtitles for a given title.')

    _ = commands.add_parser(
        'unzip',
        parents=[title, destination, _open, _ask, _group, lngs],
        aliases=['x', 'ex'],
        help='Extract downloaded title.')

    info = commands.add_parser(
        'info',
        parents=[verbose],
        aliases=['i'],
        help='Show Useful Information.')

    config = commands.add_parser(
        'config',
        parents=[],
        aliases=['cfg'],
        help='Set or reset options in user.conf')

    clean = commands.add_parser(
        'clean',
        parents=[title, lngs, ask],
        help='Delete downloaded subtitles for a title.')

    _ = commands.add_parser(
        'backup',
        parents=[destination, open_after],
        aliases=['b'],
        help='Backup cache folder as a zip file.')

    commands.add_parser(
        'login',
        parents=[],
        aliases=['l'],
        help='Login and get required cookies values.')

    commands.add_parser(
        'update',
        parents=[],
        aliases=['u'],
        help=f"Update {const.PROGRAMNAME} if a new version is available.")

    history = commands.add_parser(
        'history',
        aliases=['h'],
        help='Show or execute previous commands.')



    config_commands = config.add_subparsers(
        dest="subcommand",
        title="subcommand",
        metavar="SUBCOMMAND")
    _ = config_commands.add_parser(
        'show',
        help="show current options.")
    set_command = config_commands.add_parser(
        'set',
        help="Set an option to a value.")
    set_command.add_argument(
        'option',
        metavar="OPTION",
        help="Option to set in form of 'section.option'")
    set_command.add_argument(
        'value',
        metavar="VALUE",
        nargs='+',
        help="Value to set")

    download.add_argument(
        '-S',
        '--simulation',
        dest='simulation',
        action='store_true',
        help='Do not download, just create empty zip files.')

    _sort = info.add_mutually_exclusive_group()
    _sort.add_argument(
        "-t",
        action='store_true',
        dest='title_sort',
        help="sort output based on titles. (only with -v)")
    _sort.add_argument(
        "-n",
        action='store_true',
        dest='number_sort',
        help="sort output based on number of files of a title. (only with -v)")
    _sort.add_argument(
        "-s",
        action='store_true',
        dest='size_sort',
        help="sort output based on size of files of a title. (only with -v)")

    # clean
    clean.add_argument(
        "-0",
        "--zero",
        dest='zero',
        action='store_true',
        help="do not delete, just empty the files.")


    history_commands = history.add_subparsers(
        dest="subcommand",
        title="subcommand",
        metavar="SUBCOMMAND")
    _ = history_commands.add_parser(
        'show',
        help="show current options.")
    run_command = history_commands.add_parser(
        'run',
        help="Run a previous command again.")
    run_command.add_argument(
        'line',
        metavar="LINE",
        type=int,
        default=1,
        help="Line number of history entries to run again.")

    for _, sp in commands.choices.items():
        sp._optionals.title = "Arguments"
    # print(download.format_help())
    return parser


class CliArgs(UserConf):
    def __init__(self, argv):
        super().__init__()
        parser = get_parser()
        logger.info("new call: 'ezsub %s'", " ".join(argv))
        args, _ = parser.parse_known_args(argv)
        self.command = args.command
        # parser.print_usage()
        if args.__contains__('title'):
            if args.title:
                self.title = " ".join(args.title)
            else:
                self.title = ''

        if args.__contains__('lngs'):
            if args.lngs:
                self.lngs = " ".join(args.lngs)

        if args.__contains__('destination'):
            if args.destination:
                self.destination = args.destination

        if args.__contains__('site'):
            if args.site:
                self.site = " ".join(args.site)

        if args.__contains__('group'):
            # TODO: find better logic
            if '-G' in str(argv):
                self.group = args.group
            else:
                self.group = args.group or self.group

        if args.__contains__('auto_select'):
            # TODO: find better logic
            if '-A' in str(argv):
                self.auto_select = args.auto_select
            else:
                self.auto_select = args.auto_select or self.auto_select

        if args.__contains__('open_after'):
            # TODO: find better logic
            if '-O' in str(argv):
                self.open_after = args.open_after
            else:
                self.open_after = args.open_after or self.open_after

        if args.__contains__('zero'):
            self.zero = args.zero

        if args.__contains__('simulation'):
            self.simulation = args.simulation

        if args.__contains__('subcommand'):
            self.subcommand = args.subcommand

        if args.__contains__('option'):
            self.option = args.option

        if args.__contains__('value'):
            self.value = " ".join(args.value)

        if args.__contains__('line'):
            if args.line < 1:
                parser.error("Line number must be an integer greater than zero")
            else:
                self.line = args.line

        if args.__contains__('verbosity'):
            self.verbosity = args.verbosity or 0

        if args.__contains__('number_sort'):
            if args.number_sort:
                self.sort = 'n'
            elif args.size_sort:
                self.sort = 's'
            else:
                self.sort = 't'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import copy
import logging
import configparser
from datetime import datetime

from ezsub import const
from ezsub.utils import to_screen

logger = logging.getLogger(__name__)


def display_configs(path):
    lines = list()
    with open(path, 'r') as config_file:
        content = config_file.read()
        for line in content.splitlines():
            if line:
                if line[0] == ';':  # skip comments
                    continue
                elif line[0] == '[':  # add an empty line before sections
                    lines.append('')
                lines.append(line)
    lines.append('')
    to_screen('')
    to_screen(f"-------user.conf----------", style="ok")
    to_screen('\n'.join(lines))
    to_screen(f"-------user.conf----------", style="ok")
    return None


def new_parser():
    parser = configparser.ConfigParser(
        allow_no_value=True,
        comment_prefixes=('#',),
        delimiters=('='),
        empty_lines_in_values=True
    )
    parser.BOOLEAN_STATES = const.BOOLEAN_STATES
    parser.optionxform = str
    return parser


def fresh_configs():
    configs = new_parser()

    YESNO = "{" + ", ".join(const.BOOLEAN_STATES.keys()) + "}"
    ALL_LNGS = "{" + ", ".join(const.SUPPORTED_LNGS) + "}"
    SITES = "{" + ", ".join(const.MIRRORS) + "}"

    configs.add_section('Defaults')
    configs['Defaults'] = {
        ';; open_after:    Open destination folder after job is done. [boolean]': None,
        ';; auto_select:   Auto select best match in title search i.e. first title. [boolean]': None,
        ';; group:         Group subtitles in folders like this: "./destination/title/language/". [boolean]': None,
        f';;                for booleans choose from {YESNO}': None,
        ';; site:          Site to download from. Use space separated values. ': None,
        f';;                supported sites: {SITES}': None,
        ';; lngs:          Language(s) you interested to. Use space separated values.': None,
        f';;                supported lngs: {ALL_LNGS}': None,
        ';; destination:   Set a path for destination folder. Can be relative.': None,
        'open_after': const.OPEN_AFTER,
        'auto_select': const.AUTO_SELECT,
        'group': const.GROUP,
        'site': const.SITE,
        'lngs': const.LNGS,
        'destination': const.DESTINATION
    }

    configs.add_section('Login')
    configs['Login'] = {
        ";; Do not change manually else you know what are you doing.": None,
        "captcha": 'run ezsub login to fill this'
    }

    configs.add_section('Update')
    configs['Update'] = {
        ";; Do not change last_check manually else you know what are you doing.": None,
        "remind_every": const.PERIOD,
        "last_check": const.TODAY_STAMP
    }

    return configs


def add_section(configs, section):
    configs.add_section(section)
    configs[section] = fresh_configs()[section]
    return configs


def reset_option(configs, section, option):
    """replace a an option in a section with its default value"""
    configs[section][option] = fresh_configs()[section][option]
    return configs


def validate_section(configs, section):
    if configs.has_section(section):
        configs = validate_options(configs, section)
    else:
        configs = add_section(configs, section)
        logger.warn(f"user.conf, section '{section}' is missing. Added with default values.")
    return configs


def validate(configs):
    for section in const.SETTINGS_SKELETON.keys():
        configs = validate_section(configs, section)
    return configs


def validate_options(configs, section):
    for option, validator_approves in const.SETTINGS_SKELETON[section].items():
        if configs.has_option(section, option):
            value = configs[section][option]
            if not validator_approves(value):
                logger.warn(f"user.conf, option '{option}' in section '{section}' is invalid. reset to default value.")
                logger.debug(f"invalid option value was: {value}")
                configs = reset_option(configs, section, option)
        else:
            logger.warn(f"user.conf, option '{option}' in section '{section}' is missing. set to default value.")
            configs = reset_option(configs, section, option)
    return configs


class UserConf:
    def __init__(self):
        self.path = const.CONFIGFILE
        self.read()

    def read(self):
        is_changed = False
        configs = new_parser()
        if self.path.exists():
            configs.read(self.path)
            configs_before = copy.deepcopy(configs)
            configs = validate(configs)
            is_changed = (configs != configs_before)
        else:
            logger.warning("file 'user.conf' does not exist. creating with default values.")
            configs = fresh_configs()
            is_changed = True

        self.configs = configs
        self.open_after = configs['Defaults'].getboolean('open_after')
        self.auto_select = configs['Defaults'].getboolean('auto_select')
        self.group = configs['Defaults'].getboolean('group')
        self.site = configs['Defaults'].get('site')
        self.lngs = configs['Defaults'].get('lngs')
        self.destination = configs['Defaults'].get('destination')
        self.captcha = configs['Login'].get('captcha')
        self.last_check = configs['Update'].get('last_check')
        self.reminder = configs['Update'].get('remind_every')
        if is_changed:
            self.write()
        return None

    def write(self):
        with open(self.path, 'w') as file_to_write:
            self.configs.write(file_to_write)
        return None

    def get_last_check(self):
        return datetime.fromtimestamp(int(self.last_check))

    def set_last_check(self, value=const.TODAY_STAMP):
        self.configs['Update']['last_check'] = value
        self.write()
        return None

    def get_captcha(self):
        return self.configs['Login']['captcha']

    def set_captcha(self, value):
        self.configs['Login']['captcha'] = value
        self.write()
        return None

    def set_option(self, section, option, value):
        changed = False
        section = section.title()
        option = option.lower()
        if section in const.SETTINGS_SKELETON.keys():
            if option in const.SETTINGS_SKELETON[section].keys():
                if value == '-':
                    self.configs = reset_option(self.configs, section, option)
                    to_screen(f"user.conf ::: option is reset successfully.", style="ok")
                    changed = True
                else:
                    if const.SETTINGS_SKELETON[section][option](value):
                        self.configs[section][option] = value
                        to_screen(f"user.conf ::: option is set successfully.", style="ok")
                        changed = True
                    else:
                        to_screen(f"user.conf ::: value '{value}' is invalid for this option. ignored.", style="warn")
            else:
                to_screen(f"user.conf ::: no option named '{option}' in section '{section}'. ignored.", style="warn")
        else:
            to_screen(f"user.conf ::: no section named '{section}'. ignored", style="warn")
        if changed:
            to_screen(f"user.conf ::: write changes to file.", style="ok")
            self.write()
        return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os

from ezsub import const
from ezsub.cache import Cache
from ezsub.errors import CacheIsEmptyError
from ezsub.utils import to_screen, get_size, get_title, machine_readable, count_children


def info(req):
    cache = Cache()
    path = cache.subtitles
    if not os.listdir(path):
        raise CacheIsEmptyError
    else:
        size = get_size(path, 'human')
        children = count_children(path, 0, generation=1)
        total = count_children(path, 0, generation=3)
        basic_info(path, size, children, total)
        to_screen()
        if req.verbosity >= 1:
            level_one(path, req.sort)
        to_screen()
    return None


LANGMAX = 15
SIZEMAX = 10
COUNTMAX = 6
SPACING = 3
BAR = 41
SYMBOL = '#'


def basic_info(path, size, children, total):
    TAB = 2
    to_screen(f"\nezsub ver {const.__version__}")
    to_screen('\n[basic info]')
    to_screen(f'{" "*TAB}Path     :  {path}')
    to_screen(f'{" "*TAB}Size     :  {size}')
    to_screen(f'{" "*TAB}Titles   :  {children}')
    to_screen(f'{" "*TAB}Subtitles:  {total}')
    return None


def level_one(path, sort_key):
    items = get_sorted_items(path, sort_key)
    tick = {'s': '', 'n': '', 't': ''}
    tick[sort_key] = SYMBOL
    headers = [
        f'{tick["n"]}Files'.rjust(COUNTMAX),
        f'{tick["s"]}Size'.rjust(SIZEMAX),
        f'{tick["t"]}Title'
    ]
    header = (' '*SPACING).join(headers)
    subheaders = ["="*COUNTMAX, "="*SIZEMAX, "="*BAR]
    subheader = (" "*SPACING).join(subheaders)
    to_screen(header)
    to_screen(subheader)
    for item in items:
        row_items = [
            str(item['n']).rjust(COUNTMAX),
            item['s'].rjust(SIZEMAX),
            item['t']
        ]
        to_screen((' '*SPACING).join(row_items))
    to_screen(subheader)
    to_screen(header)
    to_screen(f'\nresults are sorted by {SYMBOL} marked column.\n', style="warn")
    return None


def get_sorted_items(path, sort_key):
    items = list()
    for child in path.iterdir():
        items.append({
            't': get_title(child),
            's': get_size(child, 'human'),
            'n': count_children(child, 0, generation=2)
        })

    def key(x):
        if sort_key == 's':
            return machine_readable(x[sort_key])
        else:
            return x[sort_key]

    reverse = False if sort_key == 't' else True
    return sorted(items, key=key, reverse=reverse)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import platform
from pathlib import Path
from datetime import datetime

__version__ = '2019.11.01'
SEMVER = '0.1.2'
CALVER = __version__
__author__ = 'Zaman'
__contact__ = '7amaaan@gmail.com'
__url__ = 'http://github.com/7aman/ezsub'
__license__ = 'MIT'


def get_root(OS):
    if OS == 'Windows':
        root = Path(os.environ['PROGRAMDATA']).joinpath('ezsub')
    elif OS == 'Linux':
        root = Path().home().joinpath('.ezsub')
    else:
        root = Path().home().joinpath('.ezsub')
    root.mkdir(parents=True, exist_ok=True)
    return root


def get_destination():
    return Path().home().joinpath('Downloads', 'ezsub').resolve()


PROGRAMNAME = 'ezsub'
OS = platform.system()
ROOT = get_root(OS)
HISTORY = ROOT.joinpath('history.txt')
CONFIGFILE = ROOT.joinpath('user.conf')
LOGFILE = ROOT.joinpath('ezsub.log')
LOGLEVEL = 'INFO'
LOGFILEMODE = 'a'
LOGFORMAT = "[%(asctime)s][%(levelname)s]{%(name)s:%(lineno)d}# %(message)s"
DESTINATION = get_destination()
MIRRORS = ['subscene', 'hastisub', 'subf2m', 'xyz']
SITE = MIRRORS[0]
AUTO_SELECT = False
OPEN_AFTER = True
GROUP = True
LNGS = 'en'
PERIOD = 7  # days delay to remind for update
BOOLEAN_STATES = {
    '0': False,
    'false': False,
    'False': False,
    'no': False,
    'No': False,
    '1': True,
    'true': True,
    'True': True,
    'yes': True,
    'Yes': True,
}
LANGUAGE_PAIRS = {
    "ar": "arabic",
    "da": "danish",
    "en": "english",
    "es": "spanish",
    "fa": "farsi_persian",
    "fr": "french",
    "he": "hebrew",
    "id": "indonesian",
    "it": "italian",
    "no": "norwegian",
    "sv": "swedish",
    "vi": "vietnamese",
    "big5": "big_5_code"
}
SUPPORTED_LNGS = list(LANGUAGE_PAIRS.keys())
TODAY = datetime.now()
TODAY_STAMP = str(int(TODAY.timestamp()))
TODAY_TIMEADD = TODAY.strftime('%Y%m%d-%H%M')


def valid_boolean(value):
    if value in BOOLEAN_STATES.keys():
        return True
    return False


def valid_site(value):
    """site value can be space separated site names"""
    sites = value.split()
    for site in sites:
        if site not in MIRRORS:
            return False
    return True


def valid_lngs_string(value):
    for lng in value.split():
        if lng not in SUPPORTED_LNGS:
            return False
    return True


def valid_destination(value):
    path = Path(value).resolve()
    if path.exists():
        if not path.is_dir():
            return False
    return True


def valid_captcha(value):
    return True


def valid_reminder(value):
    try:
        value = int(value)
        return value > 0
    except ValueError:
        return False


def valid_timestamp(value):
    try:
        value = int(value)
        return value > 0
    except ValueError:
        return False


SETTINGS_SKELETON = {
    'Defaults': {
        'open_after': valid_boolean,
        'auto_select': valid_boolean,
        'group': valid_boolean,
        'site': valid_site,
        'lngs': valid_lngs_string,
        'destination': valid_destination
    },
    'Login': {
        'captcha': valid_captcha
    },
    'Update': {
        "remind_every": valid_reminder,
        "last_check": valid_timestamp
    }
}

# requests timeout
TIMEOUT = 5
RETRY = 3
SIGNS = [
    "Temporary unavailable",
    "Request Timeout",
    "please retry a few minutes later.",
    'Backend server error',
    'many requests'
]
BAD = [
    "Bad request"
]
MAX_WORKERS = 8


# colors and text formats
class Style:
    END = '\033[0m'
    BOLD = '\033[1m'
    ITALIC = '\033[3m'
    UNDERSCORE = '\033[4m'
    URL = UNDERSCORE
    REVERSE = '\033[7m'
    TITLE = REVERSE
    CROSSED = '\033[9m'
    OVERLINED = '\033[53m'
    BLUE = '\033[34m'
    RED = '\033[91m'
    FAIL = RED
    ERROR = RED
    GREEN = '\033[92m'
    SUCCESS = GREEN
    OK = GREEN
    YELLOW = '\033[93m'
    WARNING = YELLOW
    WARN = YELLOW
    INFO = '\033[94m'

    def __init__(self):
        if OS == "Windows":
            os.system('color')

    def demo(self):
        print("------start demo----")
        variables = dir(self)
        for a in variables:
            try:
                print(getattr(self, a) + a + self.END)
                print()
            except:
                break
        print("------stop demo----")

    def render(self, styles, text):
        ss = []
        for s in styles.split(';'):
            if hasattr(self, s.upper()):
                ss.append(getattr(self, s.upper()))
        ss.append(str(text))
        ss.append(self.END)
        return "".join(ss)


class Curser:
    LU = '\033[F'
    SAVE = '\033[s'
    RESTORE = '\033[u'
    CFH = '\033[0K'
    CTH = '\033[1K'
    CL = '\033[2K'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import subprocess
from pkg_resources import parse_version

import requests

from ezsub import const
from ezsub.conf import UserConf
from ezsub.utils import to_screen


def update(just_remind=False):
    if just_remind:
        remind_to_update()
        return None
    current = const.__version__
    to_screen(f"Checking for {const.PROGRAMNAME} update. It take some seconds.")
    to_screen(f'    Current: {current}')
    to_screen(f'    Latest : ', end='')
    remote = remote_version()
    if remote:
        to_screen(f'{remote}', style="ok")
        if parse_version(remote) > parse_version(current):
            to_screen(f'There is a new version.', style="warn")
            install(remote, action='Upgrade')
        elif parse_version(remote) < parse_version(current):
            to_screen(f'You are ahead of the latest version.', style="warn")
            install(remote, action='Downgrade')
        else:
            to_screen(f'You are using the latest version.')
        config = UserConf()
        config.set_last_check()
    else:
        to_screen('unknown', style="red")
        to_screen(f"{const.PROGRAMNAME} can not reach pypi api server", style="warn")
    return None


def install(remote, action='Upgrade'):
    answer = input(f'{action}? [y]/n: ') or 'y'
    if answer.lower() == 'y':
        to_screen(f"Installing {const.PROGRAMNAME} version {remote} silently...", end='')
        subprocess.call([
            sys.executable,
            '-m', 'pip', 'install', '--user', '-q',
            f'{const.PROGRAMNAME}=={remote}'], stdout=None)
        to_screen("Done!", style="ok")
    else:
        to_screen('skipped.', style="warn")
    return None


def remote_version():
    TIMEOUT = const.TIMEOUT * 2  # pypi is a bit slower
    url = f"https://pypi.org/pypi/{const.PROGRAMNAME}/json/"
    try:
        r = requests.get(url, timeout=TIMEOUT).json()
        return r['info']['version']
    except requests.exceptions.ConnectTimeout:
        return False
    except requests.exceptions.ConnectionError:
        return False
    return False


def remind_to_update():
    configs = UserConf()
    days_past = (const.TODAY - configs.get_last_check()).days
    if days_past > int(configs.reminder):
        to_screen("Update Reminder:", style="bold;italic;warn")
        to_screen(f"\rIt's been ", style="italic", end='')
        to_screen(f"{days_past} ", style="warn", end='')
        to_screen(f"days or more since last update check.", style="italic")
        to_screen(f"Check if there is a new version available with: ", style="italic", end='')
        to_screen(f"{const.PROGRAMNAME} update", style="ok")
        to_screen()
    return None
